// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package xflags

import (
	"github.com/stretchr/testify/assert"
	"testing"
)

func TestUsage(t *testing.T) {

	commands := New("root", CmdTest1{})
	args := []string{"-h"}

	commands.Parse(args)

	assert.True(t, commands.HelpRequested())

	usage := commands.GetDefaults()
	assert.Equal(t, "  -a\tMessage A\n  -x int\n    \tMessage X\n", usage)

}

type TestDataStruct struct {
}

func (s *TestDataStruct) Copy(_ map[string]any) {

}

func TestExecuteTypeStringIsNotSupported(t *testing.T) {

	instance := execute(CmdTest1{}, &TestDataStruct{}, "test", []string{"-a", "hello", "-x", "1"})

	err := instance.Errors()
	assert.True(t, instance.HasErrors())
	assert.Equal(t, 1, len(err))

	e := err[0]
	assert.Equal(t, "missing command", e.Error())

}

func TestExecuteHelp(t *testing.T) {
	instance := execute(CmdTest1{}, &TestDataStruct{}, "test", []string{"-h"})
	assert.True(t, instance.HelpRequested())
}

func TestNewIntWithError(t *testing.T) {

	// two is not a struct
	commands := New("root", 2)
	assert.True(t, commands.HasErrors())

}

func TestNew(t *testing.T) {

	c := struct {
		Verbose bool `short:"v" long:"verbose" description:"Show verbose debug information"`
	}{
		Verbose: false,
	}

	commands := New("root", c)
	assert.False(t, commands.HasErrors())

}

type CmdTest1 struct {
	A    bool `short:"a" description:"Message A"`
	Sub1 struct {
		B    bool `short:"b" description:"Message B"`
		Sub2 struct {
			C    bool `short:"c" description:"Message C"`
			Sub3 struct {
				D bool `short:"d" description:"Message D"`
			} `command:"sub3"`
		} `command:"sub2"`
	} `command:"sub1"`
	aa int `short:"x" description:"Message X"`
}

func TestCommand2(t *testing.T) {

	commands := New("root", CmdTest1{})
	args := []string{"-a", "sub1", "-b", "sub2", "-c", "sub3", "-d"}

	commands.Parse(args)

	assert.False(t, commands.HasErrors())
	if commands.HasErrors() {
		t.Log(commands.Errors())
	}

	assert.True(t, CmdTest1(commands.GetValues()).A)
	assert.True(t, CmdTest1(commands.GetValues()).Sub1.B)
	assert.True(t, CmdTest1(commands.GetValues()).Sub1.Sub2.C)
	assert.True(t, CmdTest1(commands.GetValues()).Sub1.Sub2.Sub3.D)

	assert.False(t, commands.HasErrors())

}
